raise "Can't run this file directly" unless defined? $lib_dir
# Net Responsibility
#
# Copyright (C) 2007-2011 Scott Severance & Robert Rosman
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# Author:
#	Scott Severance <http://www.scottseverance.us>

require 'enumerator'
require 'sqlite3'
require "#{$lib_dir}my_date"

# @logfile is an SQLite3 database class
module DB
	def create_database(filename)
		begin
			db = SQLite3::Database.new filename
			events  = "CREATE TABLE IF NOT EXISTS events (type TEXT, date INT, "
			events += "time INT);"
			urls    = "CREATE TABLE IF NOT EXISTS urls (event_id INT, "
			urls   += "hostname TEXT, path TEXT, warning TINYINT);"
			starts  = "CREATE TABLE IF NOT EXISTS starts_and_stops (event_id "
			starts += "INT, method TEXT);"
			reports = "CREATE TABLE IF NOT EXISTS reports (date INT, time INT, "
			reports += "completed TEXT);"
			
			sql = [events,urls,starts,reports].each do |s|
				raise "Invalid SQL: #{s}" unless db.complete? s
			end.join("\n")
			db.execute_batch(sql)
			
			["ALTER TABLE urls ADD warning TINYINT", "ALTER TABLE reports ADD type TEXT"].each do |qry|
				begin
					db.execute(qry)
				rescue SQLite3::SQLException => e
					raise e unless e.to_s =~ /^duplicate column name/
				end
			end
				
		rescue
			File.delete filename
			raise
		ensure
			db.close unless db.closed?
		end
	end
	
	def log_url(time, url, warning = false)
		hostname = url.gsub(/\'/, '\\\\'+'\'').gsub(Regexp.new("^[a-zA-Z]+://([^/]+)(/.*)$"), '\1')
		path = $2
		qry  = "INSERT INTO events(type,date,time) VALUES(%s,%s,%s);" %
																	[q('url'),
																	 q(time.compact_date),
																	 q(time.compact_time)]
		result = @logfile.query qry
		result.close
		w = warning ? 1 : 0
		qry  = "INSERT INTO urls(event_id,hostname,path,warning) VALUES(%s,%s,%s,%s);" % 
																	[q(@logfile.last_insert_row_id),
																	 q(hostname),
																	 q(path),
																	 q(w)]
		result = @logfile.query qry
		result.close
	end
	
	def log_startup(time, type = 'start')
		qry  = "INSERT INTO events(type,date,time) VALUES(%s,%s,%s);" %
																	[q(type),
																	 q(time.compact_date),
																	 q(time.compact_time)]
		result = @logfile.query qry
		result.close
		qry  = "INSERT INTO starts_and_stops(event_id) VALUES(%s);" %
																	[q(@logfile.last_insert_row_id)]
		result = @logfile.query qry
		result.close
	end
	
	def log_reset(time)
		log_startup(time,'reset')
	end
	
	def log_shutdown(time, method)
		qry = "INSERT INTO events(type,date,time) VALUES(%s,%s,%s);" %
																	[q('shutdown'),
																	 q(time.compact_date),
																	 q(time.compact_time)]
		result = @logfile.query qry
		result.close
		qry = "INSERT INTO starts_and_stops(event_id,method) VALUES(%s,%s)" %
																	[q(@logfile.last_insert_row_id),
																	 q(method)]
		result = @logfile.query qry
		result.close
	end
	
	def log_report_start(time, type = "manual")
		qry = "INSERT INTO reports(date,time,completed,type) VALUES(%s,%s,%s,%s);" %
																		[q(time.compact_date),
																		 q(time.compact_time),
																		 q('false'),
																		 q(type)]
		result = @logfile.query qry
		result.close
		@logfile.last_insert_row_id
	end
	
	def log_report_finish(report_id)
		qry = "UPDATE reports SET completed = %s WHERE rowid = %s;" %
																		[q('true'),
																		 q(report_id)]
		result = @logfile.query qry
		result.close
	end
	
	def last_completed_report(return_raw = false, limit = 1, only_real = true)
		qry  = "SELECT date,time,type FROM reports WHERE completed == 'true' "
		qry += "AND (type == 'scheduled' OR type == 'manual')" if only_real
		qry += "ORDER BY date DESC, time DESC LIMIT %s;" % [q(limit)]
		if return_raw
			a = @logfile.query(qry)
			if limit == 1
				n = a.to_a
			else
				n = []
				a.each {|x| n.push(x.to_a)}
			end
			a.close
			return n
		else
			raise "This code hasn't been tested yet."
			tmp = @logfile.query qry
			d = nil
			t = nil
			tmp.each do |x|
				d = tmp[0]
				t = tmp[1]
			end
			tmp.close
			return DateTime.uncompact(d.to_s + t.to_s)
		end
	end
	
	def rotate_log
		tmp = last_completed_report(true, 1, false)
		d = nil
		t = nil
		tmp.each do |x|
			d = x[0]
			t = x[1]
			break
		end
		qry  = "SELECT rowid FROM events WHERE (date < #{d} OR (date = #{d} "
		qry += "AND time <= #{t}));"
		begin
			n = @logfile.query(qry)
			a = n.to_a # For some reason, this has to be the first thing executed after the query, otherwise n.to_a returns the empty array.
			Syslog.debug(:obj => n, :file => __FILE__, :line => __LINE__)
			Syslog.debug(:obj => a, :file => __FILE__, :line => __LINE__)
		ensure
			n.close if defined? n.close
		end
		['event_id','rowid'].each do |column|
			b = []
			a.each {|x| b.push "#{column} = #{x}"}
			b.each_slice(500) { |x| delete_log_entries x, column }
		end
		#begin
			#n = @logfile.query("SELECT count() FROM reports;")
			#num_reports = n.to_a
			#Syslog.debug(:obj => n, :file => __FILE__, :line => __LINE__)
			#num_reports = num_reports[0][0].to_i
			#if num_reports > 365
				#excess = num_reports - 365
				#@logfile.query "DELETE FROM reports ORDER BY date,time LIMIT #{excess};"
			#end
		result = @logfile.query "DELETE FROM reports WHERE date<#{(DateTime.now - 90).compact_date};"
		result.close
		#ensure
			#n.close if defined? n.close
		#end
	end
	
	def get_dates
		a = @logfile.query "SELECT DISTINCT date FROM events ORDER BY date;"
		b = []
		a.each {|x| b.push(:date => x[0])}
		a.close
		b.each_index {|x| b[x][:date] = DateTime.uncompact_date b[x][:date]}
		b

	end

	def get_hostnames(date)
		qry  = "SELECT DISTINCT hostname FROM urls JOIN events ON event_id = events.rowid "
		qry += "WHERE events.date = %s " % q(date.compact_date) if date
		qry += "ORDER BY hostname;"
		b = []
		a = @logfile.query qry
		a.each {|x| b.push(:hostname => x[0])}
		a.close
		b
	end

	def get_subdomains(topdomain)
		qry  = "SELECT DISTINCT hostname FROM urls WHERE hostname LIKE '\%" + topdomain + "' "
		qry += "ORDER BY hostname;"
		b = []
		a = @logfile.query qry
		a.each {|x| b.push(x[0]) if x[0].match(Regexp.new('^(.*\.)?' + topdomain))} #Since SQLite3 doesn't support regexp by default
		a.close
		b
	end

	def get_paths(hostname = false)
		qry  = "SELECT date, time, path, hostname FROM urls JOIN events ON event_id = events.rowid "
		qry += "WHERE hostname = %s " % q(hostname) if hostname
		qry += "ORDER BY date, time;"
		a = @logfile.query qry
		b = []
		a.each {|x| b.push(:time => "%s - %s" % [DateTime.uncompact_date(x[0]), DateTime.uncompact_time(x[1])], :path => x[2], :hostname => x[3])}
		a.close
		b
	end
	
	def get_warnings(time = false)
		qry  = "SELECT date, time, path, hostname FROM urls JOIN events ON event_id = events.rowid "
		qry += "WHERE warning = 1 "
		qry += "AND date = %d AND time > %d " % [DateTime.now.compact_date, DateTime.now.compact_time-3600] if time
		qry += "ORDER BY date, time;"
		a = @logfile.query qry
		b = []
		a.each {|x| b.push(:time => "%s - %s" % [DateTime.uncompact_date(x[0]), DateTime.uncompact_time(x[1])], :path => x[2], :hostname => x[3])}
		a.close
		b
	end
	
	def get_starts(date)
		qry  = "SELECT time FROM events WHERE date = %s " % q(date.compact_date)
		qry += "AND type = 'start' ORDER BY time;"
		a = @logfile.query qry
		b = []
		a.each {|x| b.push(:time => x[0])}
		a.close
		b.each_index {|x| b[x][:time] = DateTime.uncompact_time b[x][:time]}
		b
	end
	
	def get_stops(type = 'shutdown')
		qry  = "SELECT time, starts_and_stops.method, date FROM events JOIN "
		qry += "starts_and_stops on events.rowid = starts_and_stops.event_id "
		qry += "WHERE type = %s " % q(type)
		qry += "ORDER BY date, time;"
		a = @logfile.query qry
		b = []
		a.each do |x|
				b.push(:time => "%s - %s" % [DateTime.uncompact_date(x[2]), DateTime.uncompact_time(x[0])],
						:method => "<b>The program was interrupted and shutdown</b>") if x[1] == "Interrupt"
		end
		a.close
		#b.each_index {|x| b[x][:time] = DateTime.uncompact_time b[x][:time]}
		b
	end
	
	def get_resets(date)
		get_stops date, 'reset'
	end
	
	# escapes values in preparation for insertion into SQLite3
	def q(str)
		if str.kind_of? Numeric
			return str.to_s
		end
		s = str.to_s.gsub(/'/,"''")
		"'#{s}'"
	end
	
	# This is factored out of the method rotate_log. It would go at line 155, where b is defined.
	def delete_log_entries(b, column)
		b = b.join(' OR ')
		unless b.length < 10
			case column
				when 'event_id'
					#b.each do |x|
					begin
						z = @logfile.query "DELETE FROM starts_and_stops WHERE #{b};"
						Syslog.debug(:obj => z, :file => __FILE__, :line => __LINE__)
					ensure
						z.close if defined? z.close
					end
					begin
						z = @logfile.query "DELETE FROM urls WHERE #{b};"
						Syslog.debug(:obj => z, :file => __FILE__, :line => __LINE__)
					ensure
						z.close if defined? z.close
					end
					#end
				when 'rowid'
					#b.each do |x|
					begin
						z = @logfile.query "DELETE FROM events WHERE #{b};"
						Syslog.debug(:obj => z, :file => __FILE__, :line => __LINE__)
					ensure
						z.close if defined? z.close
					end
					#end
				else raise "No action defined for column '#{column}'"
			end
		end #unless
	end
end
