raise "Can't run this file directly" unless defined? $lib_dir
# Net Responsibility
#
# Copyright (C) 2007-2011 Scott Severance & Robert Rosman
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# Author:
#	Scott Severance <http://www.scottseverance.us>

#require "#{$lib_dir}logfile"
require "#{$lib_dir}options"
require "#{$lib_dir}modules/db"
require "#{$lib_dir}my_date"
require "#{$lib_dir}filter"
require "#{$lib_dir}server_connection"
require "cgi"

class Report
	include DB
	
	def initialize(db,options = nil)
		options = Options.new unless options
		@options = options
		@logfile = db
		raise "@logfile must be an open SQLite3 database" if @logfile.closed?
		#@logfile = Logfile.new @options
		dates = get_dates
		dates.each_index {|x| dates[x][:date] = DateTime.parse(dates[x][:date])}
		@dates = []
		dates.each {|x| @dates.push(x[:date])}
	end
	
	def warning_report()
		filter = Filter.new @options
		warnings = {}
		whitelist = {}
		matched_hostnames = []
		improve_data = []
		STDOUT.sync = true

		puts "\nLooking for bad keywords in the paths:"
		a = get_warnings
		a.each do |x|
			filtered = filter.token_match(x[:hostname]+x[:path])
			if filtered
				whitelist_matched = filter.whitelist_match(x[:hostname])
				if whitelist_matched
					push_match(whitelist, filtered, x[:time], whitelist_matched["match"])
					print "W"
				else
					improve_data += to_improve(filtered)
					push_match(warnings, filtered, x[:time])
					print "X"
				end
			else
				print "-"
			end
		end
		
		if not improve_data.empty? and @options.improve_data
			sc = Server_connection.new @options
			sc.add_improve_data(improve_data)
		end
		
		whitelist = whitelist.empty? ?
			[:folder => "Whitelist", :empty=>"No matches with blacklist <i>and</i> whitelist"] : 
				hits_to_arrayTree("Whitelist", whitelist)
		warnings = warnings.empty? ? 
			[:folder => "Warnings", :empty=>"No matches found (which is good)"] : 
				hits_to_arrayTree("Warnings", warnings)
		return {"warnings" => warnings, "whitelist" => whitelist}
	end

	def history(include_paths = true)
		h = get_hostnames false
		hostnames = []
		h.each  do |x| 
			#No subdomain for ip addresses
			x[:hostname].match(/^\d+\.\d+\.\d+\.\d+(:\d*)?$/) ? 
				hostnames.push(x[:hostname]) : 
				hostnames.push(x[:hostname].gsub(/(.*\.)(.*\.)(.*)$/, '\2\3'))
		end
		hostnames.uniq!
		hostnames.sort!
		t = [:folder => "Internet History"]
		hostnames.each do |topdomain|
			sub = get_subdomains topdomain
			s = [:folder => topdomain]
			sub.each do |subdomain|
				if include_paths
					paths = get_paths subdomain
					p = [:folder => subdomain]
					paths.each do |hit|
						p.push("%s%s¤http¤%s" % [subdomain, hit[:path], hit[:time]])
					end
					s.push(p)
				else
					s.push("%s¤http¤" % subdomain)
				end
			end
			t.push(s)
		end
		t.push(:empty => "Found no recorded Internet history") if t.empty?
		t
	end  
	
	def make_attached_report(input)
		open(@options.attachfile, "w") do |w|
			w.write(render_template("attached", {
				"treeContent" => to_javascript(input, "Report"),
				"title" => "%s's Net Responsibility Report" % @options.name,
				"server" => @options.server
			}))
		end
		compress_attached_report if @options.compress_attached_report or File.size(@options.attachfile) > (10*1024*1024)
	end
	
	def compress_attached_report()
		compressed = @options.attachfile.gsub(/\.html?$/, ".zip")
		zip_command = "zip -jm %s %s" % [compressed, @options.attachfile]
		@options.attachfile = compressed if `#{zip_command}`
	end
	
	def startup_report(dates = @dates)
		event_report("start",dates)
	end
	
	def shutdown_report()
		output = [:folder => "Shutdowns"]
		stops = get_stops
		if stops.empty? 
			output.push(:empty => "The program has not been interrupted.")
		else
			stops.each_index {|x| output.push("%s¤¤%s" % [stops[x][:method], stops[x][:time]])}
		end
		return output
	end
	
	def reset_report(dates = @dates)
		event_report('reset', dates)
	end
	
	def full_report(dates = @dates)
		body = []
		attached = []		
		if @options.testmail == true
			return render_template("testMail", {"name" => @options.name})
		else
			parts = @options.report_parts + @options.attached_report_parts
		
			black_and_white_warnings = warning_report if parts.include?("warnings") or parts.include?("whitelist")
			warnings = black_and_white_warnings["warnings"] if parts.include?("warnings")
			whitelist = black_and_white_warnings["whitelist"] if parts.include?("whitelist")
			history_hostnames = history if parts.include?("history_hostnames")
			history_paths = history(true) if parts.include?("history_paths")
			shutdowns = shutdown_report if parts.include?("shutdowns")
		
			@options.report_parts.each do |html_part|
				body.push(eval html_part) unless html_part == nil
			end
			
			unless @options.attached_report_parts == [nil]
				unless @options.testmail == true
					@options.attached_report_parts.each do |attached_part|
						attached.push(eval attached_part)
					end
				end
				make_attached_report(attached)
				body.push([:folder => "Attached report", :empty => "View the attached file (%s) for a better report." % @options.attachfile.gsub(/(.*\/)(.*)/, '\2')]) if File.exists? @options.attachfile
			end
			return report_header(warnings) + to_html(body)
		end
	end

	def display_latest_reports()
		d = last_completed_report(true, @options.latestreports, false)
		if d.empty?
			puts "Couldn't find any earlier reports (at least no completed)."
		else
			puts "\n   Date         Time       Type"
			d.each {|x, time, type| puts DateTime.uncompact_date(x.to_s) + '    ' + DateTime.uncompact_time(time.to_s) + (type ? '    ' + type : '')}
		end
		puts "\nNext report: #{DateTime.uncompact_date((d[0][0].to_i + @options.report_frequency).to_s)}\n\n"
		exit!(0)
	end
	
	private

	def hits_to_arrayTree (folder, hits)
		arrayTree = [:folder => folder]
		hits = hits.to_a()
		hits.sort! {|a,b| a[0]<=>b[0]}
		hits.each {|cat, hitArray| arrayTree.push(push_category(cat, hitArray))}		
		return arrayTree
	end

	def push_category(last_category, input)
		output = [:folder => last_category]
		input.each {|transfer| output.push(transfer)}
		return output
	end
	
	def push_match(push_hash, filtered, xtime, match_words = filtered["match"])
		match_words.each_pair do |words, cat|
			key = cat=="Whitelist" ? "<b>%s</b>" % words : "<b>%s</b> (%s)" % [words, cat]
			if not push_hash.has_key?(key)
				push_hash[key] = []
			end
			push_hash[key].push("%s¤http://%s¤%s" % [filtered["boldURL"], filtered["URL"], xtime])
		end
	end
	
	def event_report(event,dates = @dates)
		output = "" 
		dates.each do |date|
			date_str = date.to_date_str
			a = (method "get_#{event}s").call date
			a.each do |evt|
				case event
					when /start|reset/
						output += "    %s: %s\n" % [get_time(evt[:time]), event.capitalize]
					when "stop"
						output += "    %s - %s: %s\n" % [date_str, get_time(evt[:time]), evt[:method]]
					else
						raise "I don't know how to handle the event #{event}."
				end
			end
		end
		output + "\n"
	end
	
	def get_time(time)
		time.to_s.gsub(/^[\S]+ ([0-9:]+) .*$/,'\1')
	end
	
	def to_javascript(input, root = false)
		output = []
		input.each do |val|
			if val.class == Array
				output.push("['%s (%d)', ['',,'folder'], %s]" % [get_folder(val), get_empty_folder(val) ? 0 : val.length-1, to_javascript(val)])
			elsif !val[:folder] and !val[:empty]
				x = val.split("¤")
				output.push("\t['%s', ['%s'],, '%s']" % [x[0], x[1], x[2]])
			elsif val[:empty]
				output.push("\t['%s', ['']]" % val[:empty])
			elsif root
				output.push("['%s (%d)', ['',,'folder'], %s]" % [root, val.length-1, to_javascript(val)])
			end
		end
		
		return (root ? "[['%s', ['',,'folder'], [\n%s]]]" % [root, output.join(",\n")] :
			"[\n%s]" % output.join(",\n"))
	end

	def to_html(input, root = false)
		output = []
		input.each do |val|
			if val.class == Array
				output.push("<table><tr><td colspan='2'><b>%s%s</b> - <a name='%s' href='#top'>back to top</a></td></tr><tr><td width='10'>&nbsp;</td><td>%s</td></tr></table>" % [get_folder(val), get_empty_folder(val) ? "" : " (%d)" % (val.length-1), get_folder(val, true), to_html(val)])
			elsif !val[:folder] and !val[:empty]
				x = val.split("¤")
				output.push(abbr_link(x))
			elsif val[:empty]
				output.push("\t%s" % val[:empty])
			elsif root
				output.push("<h1>%s (%d)</h1>%s" % [root, val.length-1, to_html(val)])
			end
		end
		
		return (root ? "<h1>%s</h1>%s" % [root, output.join] : output.join("<br>"))
	end

	def to_improve(raw)
		output = []
		raw["match"].each_pair do |words, cat|
			output.push([cat, words, raw["boldURL"], raw["URL"], abbr_link(raw["boldURL"], false)].join("¤"))
		end
		return output
	end

	def get_folder(input, gs = false)
		gs = gs ? Regexp.new(/[\?\(\)\|\/\s]|<\/?b>/) : ""
		input.each {|val| return val[:folder].gsub(gs, "") if val[:folder]}
		return false
	end

	def get_empty_folder(input)
		input.each do |val|
			unless val.class == Array
				return true if val[:empty]
			end
		end
		return false
	end
	
	def report_header(warnings)	
		flags = []
		terms = ""
		if warnings != nil
			warnings.each do |folder|
				terms += "<li><a href='#%s'>%s</a> (%d)</li>" % [get_folder(folder, true), get_folder(folder), folder.length-1] if folder.class == Array
			end
			flags.push("warnings") unless get_empty_folder(warnings)
		end
		terms = "<br><b>Suspicious:</b><ul>%s</ul>" % terms unless terms == ""
		return @options.instant ? 
			render_template("instant", {
				"name" => @options.name,
				"suspicious" => terms
			}) :
			render_template("header", {
				"name" => @options.name,
				"suspicious" => terms
			}, flags)
	end
	
	def abbr_link(url, return_link = true)
		a = return_link ? CGI::unescape(url[0]) : CGI::unescape(url)
		a.gsub!(/(^[^\/]*\/)([^<>]*)([^<>]{25}<b>)/, '\1 ... \3')
		a.gsub!(/(<\/b>[^<>]{25})([^<>]*)([^<>]{25}<b>)/, '\1 ... \3')
		a.gsub!(/(<\/b>[^<>]{25})([^<>]*$)/, '\1 ...')
		a = "[<a title='Be safe!' href='%s'>Go to URL</a>] %s" %  [url[1], a] if return_link
		return a
	end
	
	def render_template(part, replace = {}, flags = [])
		output = ""
		begun = false
		template = File.open("#{$lib_dir}/report.tmpl", "r")
		while line = template.gets
			begun = true if line.match(/^<!--\##{part}-->$/)
			if begun
				break if line.match(/^<!--\#[a-zA-Z]+-->$/) and not line.match(/^<!--\##{part}-->$/)
				line.gsub!(/\{([a-zA-Z]+)\}/) {|m| replace[$1]}
				flags.each { |f| line = "" if line.match(/^(<!--#{f})|(#{f}-->)$/)}
				output += line
			end
		end
		template.close
		return output
	end
end
