raise "Can't run this file directly" unless defined? $lib_dir
# Net Responsibility
#
# Copyright (C) 2007-2011 Scott Severance & Robert Rosman
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# Author:
#	Scott Severance <http://www.scottseverance.us>

require 'yaml'
require "net/http"

class Server_connection
	def initialize(options, user = "")
		@options = options
		@user = user
	end

	def update_config_blacklist()
		downloaded = Net::HTTP.post_form(URI.parse("http://#{@options.server}/request/downloaded.php"),
													{'online_user'=>@user, 'mac'=>@options.mac})
		downloaded = YAML::load(downloaded.body)
		download_config if modified_file(@options.config, downloaded['config'])
		download_blacklists if modified_file("#{$lib_dir}lists.lst", downloaded['blacklist'])
	end
	

	def set_config(user = @options.setusername, password = @options.setpassword)
		small_config = Net::HTTP.post_form(URI.parse("http://#{@options.server}/request/add_mac.php"),
							{'online_user'=>user, 'online_password'=>password, 'mac'=>@options.mac, 'version'=>@options.version})

		if small_config.body =~ /^ERROR/
			raise small_config.body
		else
			print "Saving the configuration.\n"
			open("#{@options.config}", "w") { |file| file.write(small_config.body)}
		end
	end
	
	def download_blacklists()
		puts "Downloading the blacklists..."
		new_blacklist = Net::HTTP.post_form(URI.parse("http://#{@options.server}/request/blacklist.php"),
										{'online_user'=>@user, 'mac'=>@options.mac, 'version'=>@options.version})
		if new_blacklist.body =~ /^ERROR/
			raise new_blacklist.body
		else
			puts "Saving."
			open("#{$lib_dir}lists.lst", "w") { |file| file.write(new_blacklist.body)}
		end
	end

	def download_config()
		puts "Downloading the configurationfile..."
		new_config = Net::HTTP.post_form(URI.parse("http://#{@options.server}/request/config.php"),
													{'online_user'=>@user, 'mac'=>@options.mac, 'testmail'=>@options.testmail, 'version'=>@options.version})
		if new_config.body =~ /^ERROR/
			raise new_config.body
		else
			puts "Saving it."
			open(@options.config, "w") { |file| file.write(new_config.body)}
		end
		return true	
	end

	def add_improve_data(warnings)
		req = Net::HTTP.post_form(URI.parse("http://#{@options.server}/request/add_improve_data.php"),
										{'warnings'=>warnings.join("\n"), "version"=>@options.version})
		puts "\n", req.body
	end
	
	def modified_file(f, downloaded)
		if File.exist? f
			f = File.open(f)
			return (f.mtime.to_i-downloaded > 15 or Time.now.to_i-f.mtime.to_i > (5184000)) ? true : false
		else
			return true
		end
	end

end
